# -*- coding: utf-8 -
#
# This file is part of uzmq. See the NOTICE for more information.
"""
Classes
-------

- ``ZMQ`` : :doc:`zmq` class
- ``ZMQPoll`` : :doc:`poll` class

"""

version_info = (0, 3, 0)
__version__ = ".".join([str(v) for v in version_info])

from uzmq.poll import ZMQPoll
from uzmq.sock import ZMQ



# -*- coding: utf-8 -
#
# This file is part of uzmq. See the NOTICE for more information.


import pyuv
import zmq
import uzmq

loop = pyuv.Loop.default_loop()

ctx = zmq.Context()
s = ctx.socket(zmq.REQ)
s.connect('tcp://127.0.0.1:5555')


i = 1

def req_handler(handle, events, errors):
    global i
    if events &  pyuv.UV_WRITABLE:
        if i < 10:
            string = "echo %s" %i
            s.send(string.encode("utf8"))
            i += 1
        else:
            handle.stop()

    if events &  pyuv.UV_READABLE:
        msg = s.recv()
        print(msg)

poll = uzmq.ZMQPoll(loop, s)
poll.start(pyuv.UV_READABLE | pyuv.UV_WRITABLE, req_handler)


s.send(b"echo 0")


loop.run()



# -*- coding: utf-8 -
#
# This file is part of uzmq. See the NOTICE for more information.

import sys
import types

# True if we are running on Python 3.
PY3 = sys.version_info[0] == 3

try:
    is_callable = callable
except NameError:
    is_callable = lambda obj: hasattr(obj, '__call__')

if PY3:
    string_types = str,
    integer_types = int,
    class_types = type,
    text_type = str
    binary_type = bytes
else:
    string_types = basestring,
    integer_types = (int, long)
    class_types = (type, types.ClassType)
    text_type = unicode
    binary_type = str


def to_bytes(s):
    if isinstance(s, binary_type):
        return s
    return s.encode('utf8')



# -*- coding: utf-8 -
#
# This file is part of uzmq. See the NOTICE for more information.

import time

import pyuv
import zmq
import uzmq

loop = pyuv.Loop.default_loop()

ctx = zmq.Context()
s = ctx.socket(zmq.PUB)
s.bind('tcp://127.0.0.1:5555')

stream = uzmq.ZMQ(loop, s)

print("waiting for subscriber to connect...")
# We need to sleep to allow the subscriber time to connect
time.sleep(1.0)
print("...done.")


print("start publishing")
for i in range(10):
    stream.write_multipart(str(i))
print("...done.")

loop.run()



# -*- coding: utf-8 -
#
# This file is part of uzmq. See the NOTICE for more information.


import pyuv
import zmq
import uzmq

loop = pyuv.Loop.default_loop()

ctx = zmq.Context()
s = ctx.socket(zmq.SUB)
s.connect('tcp://127.0.0.1:5555')

s.setsockopt(zmq.SUBSCRIBE, b"")

stream = uzmq.ZMQ(loop, s)

def display(handle, msg, err):
    print(msg)

stream.start_read(display)
loop.run()



# -*- coding: utf-8 -
#
# This file is part of uzmq. See the NOTICE for more information.

"""
ZMQPoll: ZMQ Poll handle

"""
import pyuv
import zmq
from zmq.core.poll import Poller

from . import util

class ZMQPoll(object):
    """\
    :param loop: loop object where this handle runs (accessible
        through :py:attr:`Poll.loop`).
    :param int socket: zmq socket
        to be monitored for readibility or writability.

    ``ZMQPoll`` ZMQPoll handles can be used to monitor any ZMQ
    sockets for readability or writability.

    .. py:attribute:: loop

        *Read only*

        :py:class:`pyuv.Loop` object where this handle runs.

    """

    def __init__(self, loop, socket):
        self.loop = loop
        self.socket = socket

        # initialize private variable
        self._timer_h = pyuv.Timer(loop)
        self._poller = Poller()
        self._callback = None
        self._started = False

    @property
    def active(self):
        """*Read only*

            Indicates if this handle is active."""
        return self._timer_h.active

    @property
    def closed(self):
        """*Read only*

            Indicates if this handle is closing or already closed."""
        return self._timer_h.closed

    def start(self, events, callback, timeout=-1):
        """\
        :param events: int
            Mask of events that will be detected. The possible
            events are `pyuv.UV_READABLE` or `pyuv.UV_WRITABLE`.

        :param callback: callable
            Function that will be called when the ``Poll`` handle
            receives events.

        :param timeout: int
            Timeoout between each poll.

        Callback signature: ``callback(poll_handle, events, errorno)``.

        Start or update the event mask of the ``ZMQPoll`` handle.
        """
        if not util.is_callable(callback):
            raise TypeError("a callable is required")

        if timeout is None or timeout<0:
            timeout = 0.01

        self._callback = callback

        z_events = 0
        if events & pyuv.UV_READABLE:
            z_events |= zmq.POLLIN

        if events & pyuv.UV_WRITABLE:
            z_events |= zmq.POLLOUT


        if self._timer_h.active:
            self._timer.stop()
            self._poller.modify(self.socket, z_events)
        else:
            self._poller.register(self.socket, z_events)

        self._timer_h.start(self._on_timeout, timeout, timeout)


    def stop(self):
        """ Stop the ``Poll`` handle. """
        self._timer_h.stop()
        self._poller.unregister(self.socket)

    def close(self, callback=None):
        """
        :param callable callback: Function that will be called after the
            ``ZMQPoll`` handle is closed.

        Close the ``ZMQPoll`` handle. After a handle has been closed no other
        operations can be performed on it.
        """
        self._timer_h.close()
        self._poller.unregister(self.socket)

        if util.is_callable(callback):
            callback(self)

    def _on_timeout(self, handle):
        # trick to use the last state. Fix a race condition

        errno = 0
        z_events = self._poller.poll(0)
        if not z_events:
            return

        for fd, evt in z_events:
            events = 0
            if evt & zmq.POLLIN:
                events |= pyuv.UV_READABLE

            if evt & zmq.POLLOUT:
                events |= pyuv.UV_WRITABLE

            self._callback(self, events, errno)



# -*- coding: utf-8 -
#
# This file is part of uzmq. See the NOTICE for more information.

import pyuv
import zmq
import uzmq


loop = pyuv.Loop.default_loop()

ctx = zmq.Context()
s = ctx.socket(zmq.REQ)
s.connect('tcp://127.0.0.1:5555')


def display(handle, msg, err):
    print(msg[0])

stream = uzmq.ZMQ(loop, s)
stream.start_read(display)

for i in range(10):
    stream.write("echo %s" % i)

loop.run()


